@@ -86,10 +86,13 @@ plot_init_x <- function(x, xlab, ylab, main, ...) {
   
   tspx <- tsp(x)
   
+  finite_x_vals <- x[is.finite(x)]
+  
   plot_init(xmin = tspx[1L],xmax = tspx[2L]+deltat(x),
             # That x window is set to be able to translate x values
             # of deltat(x)/2 on the right
-            ymin = min(x,na.rm = TRUE), ymax = max(x,na.rm = TRUE),
+            ymin = min(finite_x_vals,na.rm = TRUE),
+            ymax = max(finite_x_vals,na.rm = TRUE),
             xlab = xlab, ylab = ylab,
             extend.x = FALSE, extend.y = TRUE, abline.x = structure(TRUE,verysmall=getOption("ts.eps")/tspx[3L]),
             main = main, ...)
@@ -197,7 +200,7 @@ window_default <- function(x,start,end) {
   
   res <- window(x,start=start,end=end,extend=TRUE)
   
-  attr(res,"coefficients") <- attr(x,"coefficients")
+  attr(res,"abline") <- attr(x,"abline")
   attr(res,"func") <- attr(x,"func")
   
   res
@@ -263,9 +266,9 @@ plotts <- function(x,show.legend,col,lty,
                      extend.x = TRUE, extend.y = TRUE,
                      abline.x=FALSE, main = main, ...)
            
-           if (!is.null(attr(x,"coefficients"))) {
-             abline(a = attr(x,"coefficients")["constant"],
-                    b = attr(x,"coefficients")[names(attr(x,"coefficients")) != "constant"],
+           if (!is.null(attr(x,"abline"))) {
+             abline(a = attr(x,"abline")["constant"],
+                    b = attr(x,"abline")["slope"],
                     col = "red",
                     lwd = 2)
            }
@@ -589,9 +592,9 @@ ggscatter <- function(object,show.legend, theme, start, end, xlab,ylab,
   
   g <- ggplot(show.legend = show.legend, ...) + xlab(xlab) + ylab(ylab)
   
-  if (!is.null(attr(object,"coefficients"))) {
-    g <- g + geom_abline(intercept = attr(object,"coefficients")["constant"],
-                         slope = attr(object,"coefficients")[names(attr(object,"coefficients")) != "constant"],
+  if (!is.null(attr(object,"abline"))) {
+    g <- g + geom_abline(intercept = attr(object,"abline")["constant"],
+                         slope = attr(object,"abline")["slope"],
                          lty = "solid", colour = "red", size = 1)
   }
   
@@ -762,4 +765,4 @@ autoplot.tscomparison <- function(object, xlab = NULL, ylab = NULL,
                 discrete_scale("linetype", "linetype_d", lty,na.translate = FALSE) +
                 ggtitle(main)
   )
-}
\ No newline at end of file
+}
