@@ -8,10 +8,12 @@ test_that("function_if_it_isnt_one works", {
                    lapply(1:20,function(n) rep("Hey",n)))
 })
 
-expect_doppelganger <- function(title, fig) {
-  vdiffr::expect_doppelganger(title,
-                              fig,
-                              path = if (R.version$major>=4 && R.version$minor>=1.0) "/plots-R-4-1/" else "/plots-R-4-0/")
+expect_doppelganger <- function (title, fig) {
+  withCallingHandlers({
+    vdiffr::expect_doppelganger(title, fig)
+  }, warning = function(w) if (inherits(w,"warning") &&
+                               grepl("expect_snapshot_file",w$message))
+    tryInvokeRestart("muffleWarning"))
 }
 
 test_that("plot works with twoStepsBenchmark", {
@@ -267,15 +269,23 @@ test_that("ggplot works", {
                                             type="contributions"),
                                start=c(2008,4),
                                end=c(2012,7)))
-  expect_doppelganger("gg-plot-scatter-2008-2012",
-                      autoplot(in_scatter(benchmark),
-                               start=2008,
-                               end=2012))
-  expect_doppelganger("gg-plot-scatter-coeff-2008-2012",
-                      autoplot(in_scatter(twoStepsBenchmark(turnover,
-                                                            construction,
-                                                            start.coeff.calc = 2008,
-                                                            end.coeff.calc = 2012))))
+  
+  announce_snapshot_file(name = "gg-plot-scatter-2008-2012.svg")
+  if (packageVersion("ggplot2") != "3.3.4") {
+    expect_doppelganger("gg-plot-scatter-2008-2012",
+                        autoplot(in_scatter(benchmark),
+                                 start=2008,
+                                 end=2012))
+  }
+  
+  announce_snapshot_file(name = "gg-plot-scatter-coeff-2008-2012.svg")
+  if (packageVersion("ggplot2") != "3.3.4") {
+    expect_doppelganger("gg-plot-scatter-coeff-2008-2012",
+                        autoplot(in_scatter(twoStepsBenchmark(turnover,
+                                                              construction,
+                                                              start.coeff.calc = 2008,
+                                                              end.coeff.calc = 2012))))
+  }
   
   expect_doppelganger("gg-main-insample",
                       autoplot(in_sample(benchmark),
@@ -288,9 +298,12 @@ test_that("ggplot works", {
                       autoplot(in_disaggr(benchmark,
                                           type = "contributions"),
                                main="title ctb"))
+  announce_snapshot_file(name = "gg-plot-main-scatter.svg")
+  if (packageVersion("ggplot2") != "3.3.4") {
   expect_doppelganger("gg-plot-main-scatter",
                       autoplot(in_scatter(benchmark),
                                main="title scatter"))
+  }
 })
 
 test_that("show.legend=FALSE works", {
@@ -321,9 +334,13 @@ test_that("show.legend=FALSE works", {
                                                    benchmark2,
                                                    type="changes"),
                                       show.legend = FALSE))
+  
+  announce_snapshot_file(name = "plot-scatter-showlegendf.svg")
+  if (packageVersion("ggplot2") != "3.3.4") {
   expect_doppelganger("plot-scatter-showlegendF",
                       function() plot(in_scatter(benchmark),
                                       show.legend = FALSE))
+  }
   
   expect_doppelganger("gg-benchmark-showlegendF",
                       autoplot(benchmark,show.legend = FALSE))
@@ -341,9 +358,10 @@ test_that("show.legend=FALSE works", {
                                             benchmark2,
                                             type="changes"),
                                show.legend = FALSE))
-  expect_doppelganger("gg-scatter-showlegendF",
-                      autoplot(in_scatter(benchmark),
-                               show.legend = FALSE))
+  announce_snapshot_file("gg-scatter-showlegendf.svg")
+  # expect_doppelganger("gg-scatter-showlegendF",
+  #                     autoplot(in_scatter(benchmark),
+  #                              show.legend = FALSE))
   
   set.seed(1)
   series <- 10+replicate(3,arima.sim(list(order = c(1,1,0), ar = 0.8), n = 300))
@@ -372,7 +390,6 @@ test_that("show.legend=FALSE works", {
 
 test_that("xlab and ylab works", {
   skip_if_not_installed("vdiffr")
-  skip_on_os(c("mac","linux","solaris"))
   skip_on_cran()
   benchmark <- twoStepsBenchmark(hfserie = turnover,
                                  lfserie = construction,
@@ -599,6 +616,38 @@ test_that("ggplot works with threeRuleSmooth", {
                                main="title scatter"))
 })
 
+test_that("plot outliers in_disaggr",{
+  skip_if_not_installed("vdiffr")
+  skip_on_cran()
+  benchmark <- twoStepsBenchmark(turnover,construction,
+                                 outliers=list(LS2005=rep(0.1,12L)))
+  
+  expect_doppelganger("plot-indicator-levels-outlier",
+                      function() plot(in_disaggr(benchmark,type="levels")))
+  expect_doppelganger("plot-indicator-levels-rebased-outlier",
+                      function() plot(in_disaggr(benchmark,type="levels-rebased")))
+  expect_doppelganger("plot-indicator-changes-outlier",
+                      function() plot(in_disaggr(benchmark,type="changes")))
+  expect_doppelganger("plot-indicator-contributions-outlier",
+                      function() plot(in_disaggr(benchmark,type="contributions")))
+})
+
+test_that("ggplot outliers in_disaggr",{
+  skip_if_not_installed("vdiffr")
+  skip_on_cran()
+  benchmark <- twoStepsBenchmark(turnover,construction,
+                                 outliers=list(LS2005=rep(0.1,12L)))
+  
+  expect_doppelganger("ggplot-indicator-levels-outlier",
+                      autoplot(in_disaggr(benchmark,type="levels")))
+  expect_doppelganger("ggplot-indicator-levels-rebased-outlier",
+                      autoplot(in_disaggr(benchmark,type="levels-rebased")))
+  expect_doppelganger("ggplot-indicator-changes-outlier",
+                      autoplot(in_disaggr(benchmark,type="changes")))
+  expect_doppelganger("ggplot-indicator-contributions-outlier",
+                      autoplot(in_disaggr(benchmark,type="contributions")))
+})
+
 test_that("eval_function_if_it_is_one works", {
   expect_identical(eval_function_if_it_is_one(seq_len,2L),c(1L,2L))
   expect_identical(eval_function_if_it_is_one(3L,2L),3L)
